@@ -4,30 +4,7 @@ use strict;
 use warnings;
 use Exporter 'import';
 
-our @OPERATIONS;
-BEGIN {
-    @OPERATIONS =
-      ( qw(OP_POWER OP_MATCH OP_NOT_MATCH OP_MULTIPLY OP_DIVIDE OP_MODULUS
-           OP_REPEAT OP_ADD OP_SUBTRACT OP_CONCATENATE OP_NUM_LT OP_NUM_GT
-           OP_NUM_LE OP_NUM_GE OP_STR_LT OP_STR_GT OP_STR_LE OP_STR_GE
-           OP_NUM_EQ OP_NUM_NE OP_NUM_CMP OP_STR_EQ OP_STR_NE OP_STR_CMP
-           OP_LOG_AND OP_LOG_OR OP_DOT_DOT OP_DOT_DOT_DOT OP_ASSIGN
-           OP_ADD_ASSIGN OP_SUBTRACT_ASSIGN OP_MULTIPLY_ASSIGN
-           OP_DIVIDE_ASSIGN OP_LOG_AND OP_LOG_OR OP_LOG_XOR OP_PLUS
-           OP_MINUS OP_LOG_NOT OP_REFERENCE OP_LOG_NOT OP_PARENTHESES
-           OP_BIT_AND OP_BIT_OR OP_BIT_XOR
-           OP_QL_S OP_QL_M OP_QL_TR OP_QL_QR OP_QL_QX OP_QL_LT OP_QL_QW
-           OP_BACKTICK OP_LOCAL
-
-           OP_FT_EREADABLE OP_FT_EWRITABLE OP_FT_EEXECUTABLE OP_FT_EOWNED
-           OP_FT_RREADABLE OP_FT_RWRITABLE OP_FT_REXECUTABLE OP_FT_ROWNED
-           OP_FT_EXISTS OP_FT_EMPTY OP_FT_NONEMPTY OP_FT_ISFILE
-           OP_FT_ISDIR OP_FT_ISSYMLINK OP_FT_ISPIPE OP_FT_ISSOCKET
-           OP_FT_ISBLOCKSPECIAL OP_FT_ISCHARSPECIAL OP_FT_ISTTY
-           OP_FT_SETUID OP_FT_SETGID OP_FT_STICKY OP_FT_ISASCII
-           OP_FT_ISBINARY OP_FT_MTIME OP_FT_ATIME OP_FT_CTIME
-           ) );
-}
+use Language::P::Opcodes qw(:all);
 
 our @EXPORT_OK =
   ( qw(NUM_INTEGER NUM_FLOAT NUM_HEXADECIMAL NUM_OCTAL NUM_BINARY
@@ -45,12 +22,12 @@ our @EXPORT_OK =
        FLAG_RX_FREE_FORMAT FLAG_RX_ONCE FLAG_RX_GLOBAL FLAG_RX_KEEP
        FLAG_RX_EVAL FLAG_RX_COMPLEMENT FLAG_RX_DELETE FLAG_RX_SQUEEZE
 
-       VALUE_SCALAR VALUE_ARRAY VALUE_HASH VALUE_SUB VALUE_GLOB
+       VALUE_SCALAR VALUE_ARRAY VALUE_HASH VALUE_SUB VALUE_GLOB VALUE_HANDLE
        VALUE_ARRAY_LENGTH
 
        DECLARATION_MY DECLARATION_OUR DECLARATION_STATE
        DECLARATION_CLOSED_OVER
-       ), @OPERATIONS );
+       %KEYWORD_TO_OP), @OPERATIONS );
 our %EXPORT_TAGS =
   ( all => \@EXPORT_OK,
     );
@@ -98,6 +75,7 @@ use constant
     VALUE_SUB          => 4,
     VALUE_GLOB         => 5,
     VALUE_ARRAY_LENGTH => 6,
+    VALUE_HANDLE       => 7,
 
     # function calls
     FLAG_IMPLICITARGUMENTS => 1,
@@ -122,10 +100,70 @@ use constant
     DECLARATION_STATE        => 4,
     DECLARATION_CLOSED_OVER  => 8,
     DECLARATION_TYPE_MASK    => 7,
-
-    map { $OPERATIONS[$_] => $_ + 1 } 0 .. $#OPERATIONS,
     };
 
+my %prototype_bi =
+  ( OP_PRINT()       => [ -1, -1, PROTO_FILEHANDLE, PROTO_ARRAY ],
+    OP_DEFINED()     => [  0,  1, PROTO_AMPER, PROTO_AMPER|PROTO_ANY ],
+    OP_RETURN()      => [ -1, -1, 0, PROTO_ARRAY ],
+    OP_UNDEF()       => [  0,  1, 0, PROTO_ANY ],
+    OP_EVAL()        => [  0,  1, PROTO_BLOCK, PROTO_ANY ],
+    OP_MAP()         => [  2, -1, PROTO_INDIROBJ, PROTO_ARRAY ],
+    map { $_ => [ 0, 1, 0, PROTO_MAKE_GLOB ] }
+        ( OP_FT_EREADABLE,
+          OP_FT_EWRITABLE,
+          OP_FT_EEXECUTABLE,
+          OP_FT_EOWNED,
+          OP_FT_RREADABLE,
+          OP_FT_RWRITABLE,
+          OP_FT_REXECUTABLE,
+          OP_FT_ROWNED,
+          OP_FT_EXISTS,
+          OP_FT_EMPTY,
+          OP_FT_NONEMPTY,
+          OP_FT_ISFILE,
+          OP_FT_ISDIR,
+          OP_FT_ISSYMLINK,
+          OP_FT_ISPIPE,
+          OP_FT_ISSOCKET,
+          OP_FT_ISBLOCKSPECIAL,
+          OP_FT_ISCHARSPECIAL,
+          OP_FT_ISTTY,
+          OP_FT_SETUID,
+          OP_FT_SETGID,
+          OP_FT_STICKY,
+          OP_FT_ISASCII,
+          OP_FT_ISBINARY,
+          OP_FT_MTIME,
+          OP_FT_ATIME,
+          OP_FT_CTIME,
+          )
+    );
+
+my %context_bi =
+  ( OP_DEFINED()     => [ CXT_SCALAR ],
+    OP_RETURN()      => [ CXT_CALLER ],
+    );
+
+my %prototype_ov =
+  ( OP_UNLINK()      => [ -1, -1, 0, PROTO_ARRAY ],
+    OP_DIE()         => [ -1, -1, 0, PROTO_ARRAY ],
+    OP_OPEN()        => [  1, -1, 0, PROTO_MAKE_GLOB, PROTO_SCALAR, PROTO_ARRAY ],
+    OP_PIPE()        => [  2,  2, 0, PROTO_MAKE_GLOB, PROTO_MAKE_GLOB ],
+    OP_CHDIR()       => [  0,  1, 0, PROTO_SCALAR ],
+    OP_RMDIR()       => [  0,  1, 0, PROTO_SCALAR ],
+    OP_READLINE()    => [  0,  1, 0, PROTO_SCALAR ],
+    OP_GLOB()        => [ -1, -1, 0, PROTO_ARRAY ],
+    OP_CLOSE()       => [  0,  1, 0, PROTO_MAKE_GLOB ],
+    OP_BINMODE()     => [  0,  2, 0, PROTO_MAKE_GLOB, PROTO_SCALAR ],
+    OP_ABS()         => [  0,  1, 0, PROTO_SCALAR ],
+    OP_WANTARRAY()   => [  0,  0, 0 ],
+    );
+
+my %context_ov =
+  (
+    );
+
 package Language::P::ParseTree::Node;
 
 use strict;
@@ -336,6 +374,7 @@ our @FIELDS = qw(flags);
 
 __PACKAGE__->mk_ro_accessors( @FIELDS );
 
+sub level { 0 }
 sub symbol_name { return $_[0]->sigil . "\0" . $_[0]->name }
 sub declaration_type { $_[0]->{flags} & Language::P::ParseTree::DECLARATION_TYPE_MASK }
 sub closed_over { $_[0]->{flags} & Language::P::ParseTree::DECLARATION_CLOSED_OVER }
@@ -582,52 +621,9 @@ use strict;
 use warnings;
 use base qw(Language::P::ParseTree::FunctionCall);
 
-my %prototype_bi =
-  ( print       => [ -1, -1, Language::P::ParseTree::PROTO_FILEHANDLE, Language::P::ParseTree::PROTO_ARRAY ],
-    defined     => [  0,  1, Language::P::ParseTree::PROTO_AMPER, Language::P::ParseTree::PROTO_AMPER|Language::P::ParseTree::PROTO_ANY ],
-    return      => [ -1, -1, 0, Language::P::ParseTree::PROTO_ARRAY ],
-    undef       => [  0,  1, 0, Language::P::ParseTree::PROTO_ANY ],
-    eval        => [  0,  1, Language::P::ParseTree::PROTO_BLOCK, Language::P::ParseTree::PROTO_ANY ],
-    map         => [  2, -1, Language::P::ParseTree::PROTO_INDIROBJ, Language::P::ParseTree::PROTO_ARRAY ],
-    map { $_ => [ 0, 1, 0, Language::P::ParseTree::PROTO_MAKE_GLOB ] }
-        ( Language::P::ParseTree::OP_FT_EREADABLE,
-          Language::P::ParseTree::OP_FT_EWRITABLE,
-          Language::P::ParseTree::OP_FT_EEXECUTABLE,
-          Language::P::ParseTree::OP_FT_EOWNED,
-          Language::P::ParseTree::OP_FT_RREADABLE,
-          Language::P::ParseTree::OP_FT_RWRITABLE,
-          Language::P::ParseTree::OP_FT_REXECUTABLE,
-          Language::P::ParseTree::OP_FT_ROWNED,
-          Language::P::ParseTree::OP_FT_EXISTS,
-          Language::P::ParseTree::OP_FT_EMPTY,
-          Language::P::ParseTree::OP_FT_NONEMPTY,
-          Language::P::ParseTree::OP_FT_ISFILE,
-          Language::P::ParseTree::OP_FT_ISDIR,
-          Language::P::ParseTree::OP_FT_ISSYMLINK,
-          Language::P::ParseTree::OP_FT_ISPIPE,
-          Language::P::ParseTree::OP_FT_ISSOCKET,
-          Language::P::ParseTree::OP_FT_ISBLOCKSPECIAL,
-          Language::P::ParseTree::OP_FT_ISCHARSPECIAL,
-          Language::P::ParseTree::OP_FT_ISTTY,
-          Language::P::ParseTree::OP_FT_SETUID,
-          Language::P::ParseTree::OP_FT_SETGID,
-          Language::P::ParseTree::OP_FT_STICKY,
-          Language::P::ParseTree::OP_FT_ISASCII,
-          Language::P::ParseTree::OP_FT_ISBINARY,
-          Language::P::ParseTree::OP_FT_MTIME,
-          Language::P::ParseTree::OP_FT_ATIME,
-          Language::P::ParseTree::OP_FT_CTIME,
-          )
-    );
-
-my %context_bi =
-  ( defined     => [ Language::P::ParseTree::CXT_SCALAR ],
-    return      => [ Language::P::ParseTree::CXT_CALLER ],
-    );
-
 sub parsing_prototype { return $prototype_bi{$_[0]->function} }
 sub runtime_context { return $context_bi{$_[0]->function} }
-sub can_implicit_return { return $_[0]->function eq 'return' ? 0 : 1 }
+sub can_implicit_return { return $_[0]->function == Language::P::ParseTree::OP_RETURN ? 0 : 1 }
 
 package Language::P::ParseTree::BuiltinIndirect;
 
@@ -645,25 +641,6 @@ use strict;
 use warnings;
 use base qw(Language::P::ParseTree::FunctionCall);
 
-my %prototype_ov =
-  ( unlink      => [ -1, -1, 0, Language::P::ParseTree::PROTO_ARRAY ],
-    die         => [ -1, -1, 0, Language::P::ParseTree::PROTO_ARRAY ],
-    open        => [  1, -1, 0, Language::P::ParseTree::PROTO_MAKE_GLOB, Language::P::ParseTree::PROTO_SCALAR, Language::P::ParseTree::PROTO_ARRAY ],
-    pipe        => [  2,  2, 0, Language::P::ParseTree::PROTO_MAKE_GLOB, Language::P::ParseTree::PROTO_MAKE_GLOB ],
-    chdir       => [  0,  1, 0, Language::P::ParseTree::PROTO_SCALAR ],
-    rmdir       => [  0,  1, 0, Language::P::ParseTree::PROTO_SCALAR ],
-    readline    => [  0,  1, 0, Language::P::ParseTree::PROTO_SCALAR ],
-    glob        => [ -1, -1, 0, Language::P::ParseTree::PROTO_ARRAY ],
-    close       => [  0,  1, 0, Language::P::ParseTree::PROTO_MAKE_GLOB ],
-    binmode     => [  0,  2, 0, Language::P::ParseTree::PROTO_MAKE_GLOB, Language::P::ParseTree::PROTO_SCALAR ],
-    abs         => [  0,  1, 0, Language::P::ParseTree::PROTO_SCALAR ],
-    wantarray   => [  0,  0, 0 ],
-    );
-
-my %context_ov =
-  (
-    );
-
 sub parsing_prototype { return $prototype_ov{$_[0]->function} }
 sub runtime_context { return $context_ov{$_[0]->function} }
 
@@ -673,7 +650,7 @@ use strict;
 use warnings;
 use base qw(Language::P::ParseTree::Overridable);
 
-sub function { 'glob' }
+sub function { Language::P::ParseTree::OP_GLOB }
 
 package Language::P::ParseTree::InterpolatedPattern;
 
